import random
import subprocess

path_to_AutoMoDe_executable = "/home/jkuckling/AutoMoDe-BT/bin/automode_main_bt"
scenario_file = "/home/jkuckling/AutoMoDe-missions/vanilla/for250sec/foraging_bt.argos"


class SequenceStarNode:

    def __init__(self, selectors):
        self.selectors = selectors

    def to_cmd(self):
        args = ["--nroot", "3"]
        args.extend(["--nchildroot", str(len(self.selectors))])
        count = 0
        for selector in self.selectors:
            args.extend(selector.to_cmd(str(count)))
            count += 1
        return args


class SelectorNode:

    def __init__(self, condition, action):
        self.condition = condition
        self.action = action

    def to_cmd(self, identifier):
        num_conditions = 1
        args = ["--n{}".format(identifier), "0", "--nchild{}".format(identifier),
                "{}".format(num_conditions + 1)]
        condition_args = self.condition.to_cmd(identifier)
        args.extend(condition_args)
        action_args = self.action.to_cmd(identifier)
        args.extend(action_args)
        return args


class ActionNode:

    def __init__(self, action, params):
        self.action_int = action
        self.params = params

    def to_cmd(self, identifier):
        i = identifier
        args = ["--n{}1".format(i), "5", "--a{}1".format(i), str(self.action_int)]
        for param in self.params:
            if param == "att" or param == "rep":
                pval = "%.2f" % self.params[param]
            elif param == "rwm":
                pval = str(self.params[param])
            else:
                print("Undefined parameter")
                pval = 0
            args.extend(["--{}{}1".format(param, i), pval])
        args.extend(["--p{}1".format(i), "0"])
        return args


class ConditionNode:

    def __init__(self, condition, params):
        self.condition_int = condition
        self.params = params

    def to_cmd(self, identifier):
        i = identifier
        args = ["--n{}0".format(i), "6", "--c{}0".format(i), str(self.condition_int)]
        for param in self.params:
            # "BlackFloor", "GrayFloor", "WhiteFloor", "FixedProbability"
            if self.condition_int == 0 or self.condition_int == 1 or self.condition_int == 2 or self.condition_int == 5:
                if param=="p":
                    pval = "%.2f" % self.params[param]
            # "NeighborsCount", "InvertedNeighborsCount"
            if self.condition_int == 3 or self.condition_int == 4:
                if param == "w":
                    pval = "%.2f" % self.params[param]
                if param == "p":
                    pval = str(self.params[param])
            args.extend(["--{}{}0".format(param, i), pval])
        return args


def evaluate_controller(controller):
    # create 10 random seeds
    seeds = [random.randint(0, 2147483647) for _ in range(0, 10)]
    # convert the controller to cmd args
    controller_cmd = controller.to_cmd()
    scores = []
    for seed in seeds:
        # prepare cmd
        args = [path_to_AutoMoDe_executable, "-n", "-c", scenario_file, "--seed", str(seed), "--bt-config"]
        args.extend(controller_cmd)
        p = subprocess.Popen(args, stderr=subprocess.PIPE, stdout=subprocess.PIPE)
        (stdout, stderr) = p.communicate()
        # Analyse result
        output = stdout.decode('utf-8')
        lines = output.splitlines()
        try:
            # print(lines[len(lines) - 1])
            score = float(lines[len(lines) - 1].split(" ")[1])
        except:
            score = -1  # Just to be sure
            print("Args: " + str(args))
            print("Stderr: " + stderr.decode('utf-8'))
            print("Stdout: " + stdout.decode('utf-8'))
            raise
        scores.append(score)
    return scores
